/**
 * Character queue implementation. Implements a buffer of
 * fixed size for storing characters. Attempts to enqueue
 * more than N characters return false, and attempts to
 * dequeue when the queue is empty return the zero character.
 *
 * @author Ryan Hall, Ben L. Titzer
 */
class Queue {
    field data: char[];
    field head: int;
    field used: int;

    constructor(size: int) {
        data = new char[size];
        head = 0;
        used = 0;
    }

    method enqueue(c: char): boolean {
        if ( full() ) return false;
	local pos = wrap(head + used);
	data[pos] = c;
	used++;
	return true;
    }

    method dequeue(): char {
	if ( used == 0 ) return '\0';
	local c = data[head];
	head = wrap(head + 1);
	used--;
	return c;
    }

    method size(): int {
	return used;
    }

    method full(): boolean {
        return used == data.length;
    }

    method empty(): boolean {
        return used == 0;
    }

    private method wrap(i: int): int {
	if ( i >= data.length ) return i - data.length;
	return i;
    }
}
